unit iEPG;

interface

uses
  SysUtils, Classes, DateUtils, FtGlobal
  , Dialogs, Controls, Forms;   //^̂߂̏ 080423MύXbyBauer

const
  // iEPGf[^擾ʒ萔
  CEPG_SUCCESSFUL = 1;   // iEPGf[^擾
  CEPG_FAILURE    = 0;   // \ʃG[
  CEPG_NOTVERSION = - 1; // ΉiEPGo[W
  CEPG_DATEERROR  = - 2; // tG[
  CEPG_STIMEERROR = - 3; // JnG[
  CEPG_ETIMEERROR = - 4; // IG[
  CEPG_PASTDATE   = - 5; // ߋ̓tw肳Ă

  // Analyze 
//  CEPG_FAILURE  = 0; // Ή̌`
  CEPG_SINGLE   = 1; // VOervO
  CEPG_MULTI    = 2; // }`ervO

  // Reg^Cv
  CCT_SINGLE_ANA  = 'application/x-tv-program-info;';               // VOAi
  CCT_SINGLE_DIG  = 'application/x-tv-program-digital-info;';       // VOfW
  CCT_MULTI_ANA   = 'application/x-multi-tv-program-info;';         // }`Ai
  CCT_MULTI_DIG   = 'application/x-multi-tv-program-digital-info;'; // }`fW

(**
 * iEPGt@C Content-type: r
 *
 * Eversion 1 ervO
 *   application/x-tv-program-info; charset=shift_jis
 *
 * Eversion 2 ervO
 *   application/x-tv-program-digital-info; charset=shift_jis
 * 
 * Eversion 1 }`ervO
 *   application/x-multi-tv-program-info; charset=shift_jis
 *
 * Eversion 2 }`ervO
 *   application/x-multi-tv-program-digital-info; charset=shift_jis
 *)

type
  TEpgData = record
    Station: string;
    StartDateTime: TDateTime;
    EndDateTime: TDateTime;
    Title: string;
    SubTitle: string;
    Text: string;
  end;

  TiEPG = class(TPersistent)
  private
    FActive: Boolean;
    FData: array of TStringList;
    FFilename: string;
    procedure SetActive(const Value: Boolean);
    procedure SetFilename(const Value: string);
    function TryStrToTimeEx(S: string; out ATime: TDateTime): Boolean;
    function GetText(const Index: Integer): string;
    function GetValues(const Index: Integer; const AName: string): string;
    function Analyze(AData: TStringList): Integer;
    function DataFactory: TStringList;
    function ReadMultiEpg: Boolean;
    procedure FreeData;
  protected
  public
    constructor Create(const Filename: string);
    destructor Destroy; override;
    procedure Close;
    function Count: Integer;
    function GetEpgData(const Index: Integer; out EpgData: TEpgData): Integer;
    function Open: Boolean;
    property Active: Boolean read FActive write SetActive default False;
    property Filename: string read FFilename write SetFilename;
    property Text[const Index: Integer]: string read GetText;
    property Values[const Index: Integer; const AName: string]: string read GetValues;
  end;

implementation

// &amp; &lt; &gt; ɖ߂܂
function HtmlDecode(S: string): string;
begin
  Result  := S;

  Result  := StringReplace(Result, '&amp;',  '&',  [rfReplaceAll, rfIgnoreCase]);
  Result  := StringReplace(Result, '&lt;',   '<',  [rfReplaceAll, rfIgnoreCase]);
  Result  := StringReplace(Result, '&gt;',   '>',  [rfReplaceAll, rfIgnoreCase]);
  Result  := StringReplace(Result, '&quot;', '"',  [rfReplaceAll, rfIgnoreCase]);
  Result  := StringReplace(Result, '&#xD7;', '~', [rfReplaceAll, rfIgnoreCase])
end;

{ TiEPG }

function TiEPG.Analyze(AData: TStringList): Integer;
var
  ContentType: string;
begin
(*
  CANA_FAILURE  = 0; // Ή̌`
  CANA_SINGLE   = 1; // VOervO
  CANA_MULTI    = 2; // }`ervO
*)
(*
  CCT_SINGLE_ANA  = 'application/x-tv-program-info;';               // VOAi
  CCT_SINGLE_DIG  = 'application/x-tv-program-digital-info;';       // VOfW
  CCT_MULTI_ANA   = 'application/x-multi-tv-program-info;';         // }`Ai
  CCT_MULTI_DIG   = 'application/x-multi-tv-program-digital-info;'; // }`fW
*)
  ContentType := Trim(AData.Values['Content-type']);
  if SameText(Copy(ContentType, 1, Length(CCT_SINGLE_ANA)), CCT_SINGLE_ANA) then
    // VO
    Result  := CEPG_SINGLE
  else if SameText(Copy(ContentType, 1, Length(CCT_SINGLE_DIG)), CCT_SINGLE_DIG) then
    // VO
    Result  := CEPG_SINGLE
  else if SameText(Copy(ContentType, 1, Length(CCT_MULTI_ANA)), CCT_MULTI_ANA) then
    // }`
    Result  := CEPG_MULTI
  else if SameText(Copy(ContentType, 1, Length(CCT_MULTI_DIG)), CCT_MULTI_DIG) then
    // }`
    Result  := CEPG_MULTI
  else
  begin
    // ΉĂȂXg[`ۂ
    Result  := CEPG_FAILURE;
    Exit;
  end;

  if Result <> CEPG_FAILURE then
    if
      (not SameText(Trim(AData.Values['version']), '1'))
      and
      (not SameText(Trim(AData.Values['version']), '2'))
      then
    begin
      // ΉĂȂo[W
      Result  := CEPG_FAILURE;
    end;
end;

procedure TiEPG.Close;
begin
  FreeData;
  FActive := False;
end;

function TiEPG.Count: Integer;
begin
  Result  := Length(FData);
end;

constructor TiEPG.Create(const Filename: string);
begin
  FActive := False;
  SetLength(FData, 0);
  FFilename := Filename;
end;

function TiEPG.DataFactory: TStringList;
begin
  Result  := TStringList.Create;
  Result.NameValueSeparator := ':';
  Result.CaseSensitive := False;
end;

destructor TiEPG.Destroy;
begin
  FreeData;
  inherited;
end;

procedure TiEPG.FreeData;
var
  i: Integer;
begin
  // FData j
  for i := 0 to Length(FData) - 1 do
    FData[i].Free;
  SetLength(FData, 0);
end;

function TiEPG.GetEpgData(const Index: Integer; out EpgData: TEpgData): Integer;
var
  DateStr, StartStr, EndStr: string;
  RecDate: TDateTime;
  EpgVer: Integer;
  Description: string;
  Performer: string;
  stf: TextFile;  //^̂߂̏ 080423MύXbyBauer
begin
  Result  := CEPG_FAILURE;
  try
    if SameText(Values[Index, 'version'], '2') or SameText(Values[Index, 'version'], '1') then
    begin
      // iEPG version 1  Version 2 ɑΉ
      if SameText(Values[Index, 'version'], '2') then
        EpgVer  := 2
      else
        EpgVer  := 1;
      case EpgVer of
      2:
        { version 2 }
        EpgData.Station   := Values[Index, 'station-name'];
      else
        { version 1 }
        EpgData.Station   := Values[Index, 'station'];
      end;
      EpgData.Title     := Values[Index, 'program-title'];
      EpgData.SubTitle  := Values[Index, 'program-subtitle'];
      // Title  SubTitle ɂ͔p |  " ܂߂邱ƂłȂ
      EpgData.Title     := StringReplace(EpgData.Title, '|', 'b', [rfReplaceAll]);
      EpgData.Title     := StringReplace(EpgData.Title, '"', 'h', [rfReplaceAll]);
      EpgData.SubTitle  := StringReplace(EpgData.SubTitle, '|', 'b', [rfReplaceAll]);
      EpgData.SubTitle  := StringReplace(EpgData.SubTitle, '"', 'h', [rfReplaceAll]);
      EpgData.Text      := Text[Index];

      // description ɑΉ
      Description := Values[Index, 'description'];
      if Description <> '' then
        EpgData.Text  := EpgData.Text + #13#10 + Description;

      // performer ɑΉ
      Performer := Values[Index, 'Performer'];
      if Performer <> '' then
        EpgData.Text  := EpgData.Text + #13#10 + Performer;

      // t
      DateStr   := Format('%s/%s/%s', [Values[Index, 'year'], Values[Index, 'month'], Values[Index, 'date']]);
      if not TryStrToDate(DateStr, RecDate) then
      begin
        // t͂Ɏs
        Result  := CEPG_DATEERROR;
        Exit;
      end;

      // 
      StartStr  := Values[Index, 'start'];
      EndStr    := Values[Index, 'end'];

      // Jn
      if not TryStrToTimeEx(StartStr, EpgData.StartDateTime) then
      begin
        // ͂Ɏs
        Result  := CEPG_STIMEERROR;
        Exit;
      end;

      // I
      if not TryStrToTimeEx(EndStr, EpgData.EndDateTime) then
      begin
        // ͂Ɏs
        Result  := CEPG_ETIMEERROR;
        Exit;
      end;

      if EpgData.StartDateTime > EpgData.EndDateTime then
      begin
        // IԂJnԂ菬̂Ŏ̓F
        EpgData.StartDateTime := RecDate + EpgData.StartDateTime;
        EpgData.EndDateTime   := IncDay(RecDate, 1) + EpgData.EndDateTime;
      end else
      begin
        // IԂJnԂ傫̂œF
        EpgData.StartDateTime := RecDate + EpgData.StartDateTime;
        EpgData.EndDateTime   := RecDate + EpgData.EndDateTime;
      end;

      // ݂̎1̗\Jn̏ꍇ
      if EpgData.StartDateTime > IncMinute(Now, 1) then
        //  iEPG f[^
        Result  := CEPG_SUCCESSFUL
      else
        // ߋ̓t
      //^̂߂̏ 080423MύXbyBauer ``
      begin
        //I݈ȍ~̏ꍇ̂
        if EpgData.EndDateTime > Now then
        begin
          if MessageDlg('ԑg͎n܂Ă܂B^悵܂H' + #10#13 +
            '(̗\ƏdȂꍇ͕̓ۏ؏o܂)', mtConfirmation, [mbYes,mbNo], 0)
            = mrYes then
          begin
            EpgData.StartDateTime := IncSecond(Now, 2);//^JnԂ𑗂点b
            //^sǂRecNow.flgt@C̗LŊmF@
            AssignFile( stf, ExtractFilePath(Application.Exename) + '\RecNow.flg' );
            Rewrite( stf );
            CloseFile( stf );
            Result  := CEPG_SUCCESSFUL;
          end else begin
            Result  := CEPG_PASTDATE;
          end;
        end else begin
          Result  := CEPG_PASTDATE;
        end;
      end;
      //^̂߂̏ 080423MύXbyBauer `܂Ł`
    end else
      // Ή iEPG f[^
      Result  := CEPG_NOTVERSION;
  except
  end;
end;

function TiEPG.GetText(const Index: Integer): string;
var
  i: Integer;
  b: Boolean;
begin
  Result := '';
  b := False;
  for i := 0 to FData[Index].Count - 1 do
  begin
    if not b then
    begin
      if FData[Index][i] = '' then b := True;
    end else
    begin
      if Result = '' then
        Result := FData[Index][i]
      else
        Result := Result + #13#10 + FData[Index][i];
    end;
  end;
  Result  := HtmlDecode(Result);
end;

function TiEPG.GetValues(const Index: Integer; const AName: string): string;
begin
  Result := HtmlDecode(Trim(FData[Index].Values[AName]));
end;

function TiEPG.Open: Boolean;
begin
  Result := False;

  // JĂ
  if FActive then Close;

  // t@CȂI
  if not FileExists(FFilename) then Exit;

  // FData  1 쐬ăt@C[h
  SetLength(FData, 1);
  FData[0]  := DataFactory;
  FData[0].LoadFromFile(FFilename);
  case Analyze(FData[0]) of
  CEPG_SINGLE:
    begin
      // VOt@C
      // Ȃ
    end;
  CEPG_MULTI:
    begin
      // }`t@C
      if not ReadMultiEpg then
      begin
        // Ή̃t@C
        FreeData;
        Exit;
      end;
    end;
  else
    begin
      // Ή̃t@C
      FreeData;
      Exit;
    end;
  end;
  
  FActive := True;
  Result  := FActive;
end;

function TiEPG.ReadMultiEpg: Boolean;
var
  Boundary: string;
  ASrc: TStringList;
  i, n: Integer;
  IsData: Boolean;
begin
  // }` EPG ǂݍ
  Result  := False;
  if Length(FData) = 0 then Exit;

  ASrc  := TStringList.Create;
  try
    // FData[0]  ASrc ɃRs[
    ASrc.Assign(FData[0]);
    // FData j
    FreeData;

    // boundary 擾
    Boundary  := Trim(ASrc.Values['boundary']);

    // ϐ
    IsData  := False;
    n       := - 1;
    // ASrc [v
    for i := 0 to ASrc.Count - 1 do
    begin
      if not IsData then
      begin
        if SameText(ASrc[i], Boundary) then
        begin
          // f[^Jn
          n := Length(FData);
          SetLength(FData, n + 1);
          FData[n]  := DataFactory;
          IsData    := True;
        end;
      end else
      begin
        if SameText(ASrc[i], Boundary) then
        begin
          // Vf[^Jn
          n := Length(FData);
          SetLength(FData, n + 1);
          FData[n]  := DataFactory;
        end
        else if SameText(ASrc[i], Boundary + '--') then
        begin
          // f[^I
          Break;
        end else
          // f[^擾
          if n >= 0 then FData[n].Add(ASrc[i]);
      end;
    end;
    Result  := Length(FData) > 0;
  finally
    ASrc.Free;
  end;
end;

procedure TiEPG.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
  begin
    if Value then
      Open
    else
      Close;
  end;
end;

procedure TiEPG.SetFilename(const Value: string);
begin
  if FFilename <> Value then
  begin
    // JĂ
    if FActive then Close;
    
    FFilename := Value;
  end;
end;

function TiEPG.TryStrToTimeEx(S: string; out ATime: TDateTime): Boolean;
var
  Data: TStringArray;
  Hour, Min: Integer;
begin
  //  S  ':' ŕ
  Data  := Split(':', S);

  if Length(Data) >= 2 then
  begin
    // "" 
    Hour  := StrToIntDef(Data[0], - 1);
    if Hour < 0 then
    begin
      // "" s
      Result  := False;
      Exit;
    end;

    // "" 
    Min   := StrToIntDef(Data[1], - 1);
    if (Min < 0) or (Min > 59) then
    begin
      // "" s
      Result  := False;
      Exit;
    end;

    // Result := TryEncodeTime(Word(Hour), Word(Min), 0, 0, ATime);
    ATime := IncHour(0, Hour);
    ATime := IncMinute(ATime, Min);
    Result  := True;
  end else
    // 2ȏɕłȂ̂͂
    Result := False;
end;

end.
